//
// Copyright 2014-2017 Cristian Maglie. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.
//
// ppc64le specific implementation, that does not use TCGETS2 or TCSETS2, as
// they are not supported by golang v1.17 for that platform circa 2021-12.

// FIXME: is a "linux" build filter needed? could *bsd be affected?
// +build ppc64le

package serial

import (
	"golang.org/x/sys/unix"
)

func (p *Port) retrieveTermSettings() (*settings, error) {
	var err error
	s := &settings{termios: new(unix.Termios)}

	if s.termios, err = unix.IoctlGetTermios(p.internal.handle, unix.TCGETS); err != nil {
		return nil, newOSError(err)
	}
	// circa 2021-12 ppc64le golang v1.17 does not support TCGETS2

	return s, nil
}

func (p *Port) applyTermSettings(s *settings) error {
	req := uint(unix.TCSETS)
	// circa 2021-12 ppc64le golang v1.17 does not support TCSETS2

	if err := unix.IoctlSetTermios(p.internal.handle, req, s.termios); err != nil {
		return newOSError(err)
	}

	return nil
}
